import time

from datetime import datetime
from netforce.model import Model, fields, get_model
from netforce.utils import get_data_path, get_file_path
from netforce.access import get_active_user,set_active_user
from netforce.access import get_active_company


class HDCase(Model):
    _name="clinic.hd.case"
    _string="HD Case"
    _audit_log=True
    _name_field="number"
    _multi_company=True
    
    def _get_duration(self,ids,context={}):
        res={}
        fmt="%Y-%m-%d %H:%M:%S"
        for obj in self.browse(ids):
            diff=datetime.strptime(obj.time_stop,fmt)-datetime.strptime(obj.time_start,fmt)
            total_time=round(diff.seconds/3600,2)
            res[obj.id]=total_time
        return res

    def _get_pay_amount(self,ids,context={}):
        res={}
        for obj in self.browse(ids):
            res[obj.id]=obj.amount
        return res

    def _get_total(self,ids,context={}):
        vals={}
        for obj in self.browse(ids):
            total=0
            due_amt=0
            fee_amt=0
            mdc_amt=0
            for line in obj.lines:
                if line.type=='fee':
                    fee_amt+=line.amount or 0.0
                elif line.type=='medicine':
                    mdc_amt+=line.amount or 0.0
                total+=line.amount or 0.0
                    
            # TODO need to check with invoice policy
            # government pay for : 
            # - fee -> create invoice with fee type on state waiting to payment
            # - fee & medicine -> create 2 invoice for fee and medicine type
            if obj.invoice_policy=='fee':
                due_amt=total-fee_amt
            elif obj.invoice_policy=='fee_mdc':
                due_amt=total-(fee_amt+mdc_amt)
            else:
                due_amt=total

            for line in obj.payment_lines:
                due_amt-=line.amount or 0.0
            vals[obj.id]={
                "total": total,
                "amount": due_amt,
                'fee_amount': fee_amt,
                'mdc_amount': mdc_amt,
            }
        return vals
    
    def _get_patient_type(self,ids,context={}):
        res={}
        for obj in self.browse(ids):
            patient=obj.patient_id
            res[obj.id]=patient.type_id.name
        return res

    _fields={
        "number": fields.Char("Number",required=True,search=True),
        "ref": fields.Char("Ref",search=True),
        "time_start": fields.DateTime("Start Time",required=True,search=True),
        "time_stop": fields.DateTime("Finish Time",required=True,search=True),
        "date": fields.Date("Date",required=True,search=True),
        "patient_id": fields.Many2One("clinic.patient","Patient",required=True,search=True),
        "patient_type": fields.Char("Service Type",function="_get_patient_type"),
        "vascular_acc": fields.Many2One("clinic.vascular.access","Vascular Ac."),
        "nurse_id": fields.Many2One("clinic.staff","Approve By", domain=[['type','=','nurse']]),
        "department_id": fields.Many2One("clinic.department", "Department",search=True),
        "wt_kg": fields.Float("Wt.kg."),
        "bp": fields.Integer("BP"),
        "mm_hg": fields.Integer("mmHG"),
        "hct": fields.Integer("Hct%(<40)",required=True),
        "hct_msg" : fields.Char(""),
        "state": fields.Selection([("draft","Draft"),('waiting_treatment','Waiting Treatment'),("in_progress","In Progress"),("completed","Finish Treatment"),('paid','Paid'),("waiting_payment","Waiting Payment"),("discountinued","Discountinued"),("cancelled","Cancelled")],"Status",required=True),
        "staffs": fields.One2Many("clinic.hd.case.staff","hd_case_id","Staffs"),
        "comments": fields.One2Many("message","related_id","Comments"), "company_id": fields.Many2One("company","Company"),
        "dialyzers": fields.One2Many("clinic.hd.case.dialyzer","hd_case_id","Dialyzers"),
        "lines": fields.One2Many("clinic.hd.case.line","hd_case_id","Lines"),
        "invoices": fields.One2Many("account.invoice","related_id","Invoices"),
        "pickings": fields.One2Many("stock.picking","related_id","Pickings"),
        "payments": fields.One2Many("account.payment","related_id","Payments"),
        "payment_lines": fields.One2Many("clinic.payment","hd_case_id","Payment Lines"),
        "expenes": fields.One2Many("clinic.hd.case.expense","hd_case_id","Expenses"),
        'visit_id': fields.Many2One("clinic.visit", "Visit"),
        'duration': fields.Integer("Duration(Hours)",function="_get_duration"),
        "total": fields.Float("Total",function="_get_total",readonly=True,function_multi=True),
        "fee_amount": fields.Float("Fee",function="_get_total",readonly=True,function_multi=True),
        "mdc_amount": fields.Float("Medicine",function="_get_total",readonly=True,function_multi=True),
        "amount": fields.Float("Due Amount",function="_get_total",readonly=True,function_multi=True),
        'fee_partner_id': fields.Many2One("partner","Contact Fee"),
        'fee_paid': fields.Boolean("Fee Paid"),
        'note': fields.Text("Note"),
        'complication': fields.Text("Complication"),
        "cycle_id": fields.Many2One("clinic.cycle","Cycle"),
        'cycle_item_id': fields.Many2One("clinic.cycle.item","Cycle Item"), # on_delete="cascade" -> will rm visit from cycle item
        'pay_amount': fields.Float("Amount",function="_get_pay_amount"),
        'pay_date': fields.Date("Pay Date"),
        'pay_account_id': fields.Many2One("account.account","Account"),
        'payment_id': fields.Many2One("account.payment","Payment"), # for print
        'dlz_id': fields.Many2One("clinic.dialyzer","Dialyzer"), # for link
        "total_doctor": fields.Integer("Total Doctor",function="get_staff",function_multi=True),
        "total_nurse": fields.Integer("Total Nurse",function="get_staff",function_multi=True),
        'doctor_id': fields.Many2One("clinic.staff","Doctor",domain=[['type','=','doctor']],function="get_staff",function_multi=True),
        'nu': fields.Char("N/U"),
        "invoice_policy": fields.Selection([("fee","Only Fee"),("fee_mdc","Fee & Medicine")],"Government pay for:"),
        "invoice_option": fields.Selection([("fee_mdc_plus","Combine Fee & Medicine"),("fee_mdc_split","Split Fee & Medicine")],"Invoice:"),
        'req_fee': fields.Integer("Request Expense"),
    }

    def _get_number(self,context={}):
        while 1:
            seq_id=get_model("sequence").find_sequence(name="Clinic HD Case")
            num=get_model("sequence").get_next_number(seq_id,context=context)
            if not num:
                return None
            user_id=get_active_user()
            set_active_user(1)
            res=self.search([["number","=",num]])
            set_active_user(user_id)
            if not res:
                return num
            get_model("sequence").increment_number(seq_id,context=context)

    _defaults={
        "state": "draft",
        "date": lambda *a: time.strftime("%Y-%m-%d"),
        "time_start": lambda *a: time.strftime("%Y-%m-%d %H:%M:%S"),
        "time_stop": lambda *a: time.strftime("%Y-%m-%d %H:%M:%S"),
        "number": _get_number,
        "company_id": lambda *a: get_active_company(),
        'wt_kg': 0.0,
        'bp': 0,
        'mm_hg': 0,
        'hct': 0,
        'hct_msg': "สามารถเบิกค่ายาสูงสุดไม่เกิน 1,125บาท ต่อ สัปดาห์",
        'fee_paid': False,
        'invoice_option': 'fee',
        'invoice_policy': 'fee',
        'req_fee': 0,
    }
    _order="date desc,number desc"

    def onchange_dialyzer(self,context={}):
        data=context["data"]
        path=context["path"]
        line=get_data_path(data,path,parent=True)
        dialyzer_id=line.get("dialyzer_id")
        if not dialyzer_id:
            return {}
        dialyzer=get_model("clinic.dialyzer").browse(dialyzer_id)
        use_time=dialyzer.use_time or 0
        max_time=dialyzer.max_use_time or 0
        if use_time > max_time:
            dialyzer.write({
                'state': 'expire',
            })
            raise Exception("%s is expired"%dialyzer.number)
        use_time+=1
        line["description"]=dialyzer.description or dialyzer.product_id.name or ""
        line["use_time"]=use_time
        line["max_use_time"]=dialyzer.max_use_time
        line["member_type"]=dialyzer.member_type
        line["dialyzer_type"]=dialyzer.dialyzer_type
        line["bid_flow_rate"]=dialyzer.bid_flow_rate
        line["ultrafittration"]=dialyzer.ultrafittration
        line["state"]=dialyzer.state
        data['dlz_id']=dialyzer.id
        return data
    
    def onchange_cycle(self,context={}):
        data=context['data']
        cycle_id=data['cycle_id']
        cycle=get_model("clinic.cycle").browse(cycle_id)
        date=data['time_start'][0:10]
        data['time_start']='%s %s'%(date, cycle.time_start)
        data['time_stop']='%s %s'%(date, cycle.time_stop)
        fmt="%Y-%m-%d %H:%M"
        diff=datetime.strptime(data['time_stop'],fmt)-datetime.strptime(data['time_start'],fmt)
        total_time=round(diff.seconds/3600,2)
        data['duration']=total_time
        #XXX get nurse team
        return data

    def onchange_patient(self,context={}):
        data=context['data']
        patient_id=data['patient_id']
        if patient_id:
            patient=get_model('clinic.patient').browse(patient_id)
            department=patient.department_id
            cycle=patient.cycle_id
            partner=patient.type_id.contact_id
            data['department_id']=department.id
            data['cycle_id']=cycle.id
            if partner:
                data['fee_partner_id']=partner.id
            else:
                data['fee_partner_id']=None
            data['patient_type']=patient.type_id.name or ""
        data['dialyzers']=[]
        doctor=patient.doctor_id
        data['staffs']=[] # XXX
        data['staffs'].append({
            'staff_id': doctor.id,
            'type': 'doctor',
            'priop': 'owner',
        })
        return data

    def onchange_line(self,context={}):
        data=context['data']
        path=context['path']
        line=get_data_path(data,path,parent=True)
        qty=line['qty'] or 0
        price=line['price'] or 0.0
        line['amount']=qty*price
        self.update_amount(data)
        return data

    def onchange_product(self,context={}):
        data=context['data']
        path=context['path']
        line=get_data_path(data,path,parent=True)
        product_id=line.get('product_id')
        prod=get_model("product").browse(product_id)
        line['uom_id']=prod.uom_id.id
        line['description']=prod.name
        qty=1
        price=prod.sale_price or 0.0
        amt=qty*price
        line['qty']=qty
        line['price']=price
        line['amount']=amt
        self.update_amount(data)
        return data
    
    def onchange_pay(self,context={}):
        data=context['data']
        pay_amount=data['pay_amount'] or 0
        amount=data['amount'] or 0
        if pay_amount > amount:
            data['pay_amount']=0
        return data

    def onchange_type(self,context={}):
        data=self.onchange_policy(context)
        return data

    def onchange_policy(self,context={}):
        data=context['data']
        inv_pol=data['invoice_policy']
        total=0.0
        fee=0.0
        mdc=0.0
        due_amt=0.0
        for line in data['lines']:
            amt=line['amount'] or 0.0
            total+=amt
            ltype=line['type']
            if ltype=='fee':
                fee+=amt
            elif ltype=='medicine':
                mdc+=amt
            elif ltype=='others':
                pass
        if inv_pol=='fee':
            due_amt=total-fee
            data['invoice_option']=''
        elif inv_pol=='fee_mdc':
            due_amt=total-fee-mdc
            data['invoice_option']='fee_mdc_split'
        else:
            due_amt=total
        data['total']=total
        data['fee_amount']=fee
        data['mdc_amount']=mdc
        data['amount']=due_amt
        return data

    def update_amount(self,data):
        total_amt=0.0
        fee_amt=0.0
        for line in data['lines']:
            price=line.get('price') or 0
            qty=line.get('qty') or 0
            amt=qty*price
            if line['type']=='fee':
                fee_amt+=amt
            else:
                total_amt+=amt
        data['total']=total_amt+fee_amt
        data['fee_amount']=fee_amt
        for line in data['payment_lines']:
            total_amt-=line['amount'] or 0.0
        data['amount']=total_amt
        return data 

    def make_payment(self,ids,context={}):
        obj=self.browse(ids)[0]
        if not obj.total:
            return
        remaining_amt=0.0
        for line in obj.lines:
            remaining_amt+=line.amount or 0.0
        for line in obj.payment_lines:
            remaining_amt-=line.amount or 0.0
        partner=obj.patient_id.partner_id
        if not partner:
            raise Exception("No contact on this patient")
        company_id=get_active_company()
        account_receivable_id=partner.account_receivable_id
        if not account_receivable_id:
            st=get_model('settings').browse(1)
            account_receivable_id=st.account_receivable_id
            if not account_receivable_id:
                raise Exception("Not found account recieveable in account setting")
        account_id=account_receivable_id.id
        if not account_id:
            raise Exception("No Account for payment")
        pay_amount=obj.pay_amount

        if context.get("amount",0):
            pay_amount=context['amount'] or 0.0
        vals={
            "partner_id": partner.id,
            "company_id": company_id,
            "type": "in",
            "pay_type": "direct",
            'date': time.strftime("%Y-%m-%d"),
           "account_id": account_id,
           'related_id': "clinic.hd.case,%s"%obj.id,
           'direct_lines': [],
        }
        vals['direct_lines'].append(('create',{
            'description': 'Payment; %s'%obj.number,
            'account_id': account_id,
            'qty': 1,
            'unit_price': pay_amount,
            'amount': pay_amount,
        }))

        payment_id=get_model("account.payment").create(vals,context={"type":"in"})
        obj.write({
            'payment_lines': [('create',{
                        'payment_id': payment_id,
                        'amount': pay_amount,
                    })],
        })
        payment=get_model('account.payment').browse(payment_id)
        payment.post()
        return {
            'next': {
                'name': 'clinic_hd_case',
                'mode': 'form',
                'active_id': obj.id,
            },
            'flash': 'Pay OK',
        }

    def cancelled(self,ids,context={}):
        obj=self.browse(ids)[0]
        obj.write({" state":"cancelled"})

    def make_invoices(self,ids,context={}):
        setting=get_model("settings").browse(1)
        currency_id=setting.currency_id.id
        account_receivable_id=setting.account_receivable_id.id
        company_id=get_active_company()
        uom=get_model("uom").search_browse([['name','ilike','%Unit%']])
        if not uom:
            raise Exception("Unit not found in uom")
        obj=self.browse(ids[0])
        if obj.invoices:
            for inv in obj.invoices:
                inv.void()

        due_date=obj.date[0:10] # XXX
        # cash, credit
        make_invoice=context.get('make_invoice',True)
        context['type']='out'
        context['inv_type']='invoice'
        fee_lines=[]
        mdc_lines=[]
        if obj.lines:
            vals={
                "type": "out",
                "inv_type": "invoice",
                "tax_type": "tax_in",
                'due_date': due_date,
                "ref": obj.number,
                "related_id": "clinic.hd.case,%s"%obj.id,
                "currency_id": currency_id,
                "company_id": company_id,
                "lines": [],
                "company_id": company_id,
            }
            partner=obj.patient_id.partner_id
            if not partner:
                raise Exception("No contact for patient %s"%obj.patient_id.name)
            vals["partner_id"]=partner.id

            for line in obj.lines:
                if line.product_id:
                    account_id=line.product_id.sale_account_id.id or account_receivable_id
                else:
                    account_id=account_receivable_id
                inv_line={
                    "product_id": line.product_id.id,
                    "description": line.description,
                    "qty": line.qty,
                    "uom_id": line.uom_id.id,
                    "unit_price": line.price,
                    "amount": line.amount,
                    'account_id': account_id,
                }

                if obj.invoice_policy=='fee':
                    if line.type=='fee':
                        fee_lines.append(inv_line)
                    else:
                        vals['lines'].append(('create',inv_line))
                elif obj.invoice_policy=='fee_mdc':
                    if line.type=='fee':
                        fee_lines.append(inv_line)
                    elif line.type=='medicine':
                        mdc_lines.append(inv_line)
                    else:
                        vals['lines'].append(('create',inv_line))
                else:
                    vals['lines'].append(('create',inv_line))

            if obj.amount and make_invoice: ## not pay yet
                get_model("account.invoice").create(vals,context)
            # check if type of product is stockable then create picking
            obj.make_pickings()
        
        vals={
            "type": "out",
            "inv_type": "invoice",
            "tax_type": "tax_in",
            'due_date': due_date,
            "ref": obj.number,
            "related_id": "clinic.hd.case,%s"%obj.id,
            "currency_id": currency_id,
            "company_id": company_id,
            "lines": [],
            "company_id": company_id,
        }
        
        if obj.invoice_option=='fee_mdc_plus':
            fee_lines+=mdc_lines 
            mdc_lines=[]

        if mdc_lines:
            inv_vals=vals.copy()
            inv_vals["partner_id"]=obj.fee_partner_id.id
            account_id=account_receivable_id
            patient_type=obj.patient_id.type
            if patient_type=='sc':
                account_id=setting.ar_sc_id.id
            elif patient_type=='uc':
                account_id=setting.ar_uc_id.id
            for mdc_line in mdc_lines:
                mdc_line['account_id']=account_id
                inv_vals['lines'].append(('create',mdc_line))
            get_model("account.invoice").create(inv_vals,context)

        if fee_lines and not obj.fee_paid:
            inv_vals=vals.copy()
            inv_vals["partner_id"]=obj.fee_partner_id.id
            account_id=account_receivable_id
            patient_type=obj.patient_id.type
            if patient_type=='sc':
                account_id=setting.ar_sc_id.id
            elif patient_type=='uc':
                account_id=setting.ar_uc_id.id

            for fee_line in fee_lines:
                fee_line['account_id']=account_id
                inv_vals['lines'].append(('create',fee_line))
            get_model("account.invoice").create(inv_vals,context)
            obj.write({
                'fee_paid': True,
            })
    
    def make_pickings(self,ids,context={}):
        obj=self.browse(ids[0])
        # no picking
        if not obj.lines:
            return

        partner=obj.patient_id.partner_id
        if not partner:
            raise Exception("Contact not for this patient")
        ship_address_id=None
        for address in partner.addresses:
            if address.type=="shipping":
                ship_address_id=address.id
                break
        if not ship_address_id:
            raise Exception("contact %s dont'have address with type shipping"%partner.name)
        pick_vals={
            "type": "out",
            "ref": obj.number,
            "related_id": "clinic.hd.case,%s"%obj.id,
            "partner_id": obj.patient_id.partner_id.id,
            "ship_address_id": ship_address_id,
            "state": "draft",
            "lines": [],
        }
        res=get_model("stock.location").search([["type","=","customer"]])
        if not res:
            raise Exception("Customer location not found")
        cust_loc_id=res[0]


        for line in obj.lines:
            prod=line.product_id
            
            if prod.type != 'stock':
                print("continue ")
                continue
            wh_loc_id=prod.location_id.id
            if not wh_loc_id:
                res=get_model("stock.location").search([["type","=","internal"]])
                if not res:
                    raise Exception("Warehouse not found")
                wh_loc_id=res[0]
            line_vals={
                "product_id": prod.id,
                "qty": 1,
                "uom_id": prod.uom_id.id,
                "location_from_id": wh_loc_id,
                "location_to_id": cust_loc_id,
            }
            pick_vals["lines"].append(("create",line_vals))
        if not pick_vals["lines"]:
            return {
                "flash": "Nothing left to deliver",
            }
        picking_obj=get_model("stock.picking")
        pick_id=picking_obj.create(pick_vals,context={"pick_type": "out"})
        pick=picking_obj.browse(pick_id)
        pick.set_done([pick_id])

    def post_invoices(self,ids,context={}):
        obj=self.browse(ids[0])
        for inv in obj.invoices:
            inv.post()
        print("Post!") 

    def do_treatment(self,ids,context={}):
        obj=self.browse(ids)[0]
        #TODO  should find dlz when confirm visit
        if not obj.dialyzers:
            raise Exception("Please input dialyzer!")
        vals={
            'state': 'in_progress',
        }
            
        # update start time
        st=get_model("clinic.setting").browse(1)
        if st.real_time:
            timenow=time.strftime("%H:%M:%S")
            date=obj.date
            vals['time_start']='%s %s'%(date,timenow)
            vals['time_stop']='%s %s'%(date,timenow)
        obj.write(vals)

    def discontinue(self,ids,context={}):
        obj=self.browse(ids)[0]
        # TODO pop to note
        obj.write({"state":"cancelled"})

    def update_usetime(self,ids,context={}):
        for obj in self.browse(ids):
            for dlz_line in obj.dialyzers:
                dlz=dlz_line.dialyzer_id
                if dlz_line.use_time < dlz.max_use_time:
                    dlz.write({
                        'use_time': dlz_line.use_time,
                    })
                elif dlz_line.use_time==dlz.max_use_time:
                    dlz.write({
                        'use_time': dlz_line.use_time,
                        'state': 'expire',
                    })
                else:
                    raise Exception("Dialyzer is expired!")
        return True

    def create_cycle_item(self,ids,context={}):
        for obj in self.browse(ids):
            cycle_item=get_model("clinic.cycle.item")
            datenow=obj.time_start[0:10]
            if not datenow:
                datenow=time.strftime('%Y-%m-%d')
            cycle_id=obj.cycle_id.id
            cycle_item_ids=cycle_item.search([['date','=',datenow],['cycle_id','=',cycle_id]])
            cycle_item_id=None
            if cycle_item_ids:
                cycle_item_id=cycle_item_ids[0]
            else:
                cycle_item_id=cycle_item.create({
                    'date': obj.date,
                    'cycle_id': obj.cycle_id.id,
                })
            obj.write({
                'cycle_item_id': cycle_item_id,
            })
        return True

    def do_expense(self,ids,context={}):
        for obj in self.browse(ids):
            # clear old expense
            for exp in obj.expenes:
                exp.delete()

            exp_lines=[]
            for inv in obj.invoices:
                exp_lines.append(('create',{
                    'date': obj.date,
                    'patient_id': obj.patient_id.id,
                    'invoice_id': inv.id,
                    'hd_case_id': obj.id,
                    'amount': inv.amount_due,
                    'state': 'waiting_matching',
                }))
            for pm in obj.payments:
                exp_lines.append(('create',{
                    'date': obj.date,
                    'patient_id': obj.patient_id.id,
                    'payment_id': pm.id,
                    'hd_case_id': obj.id,
                    'amount': pm.amount_total,
                    'state': 'waiting_matching',
                }))
            obj.write({
                'expenes': exp_lines,
            })

    def complete(self,ids,context={}):
        obj=self.browse(ids)[0]
        obj.make_invoices()
        obj.post_invoices()
        obj.update_usetime()
        obj.create_cycle_item()
        obj.do_expense()
        vals={
            "state":"waiting_payment", # for government
        }
        st=get_model("clinic.setting").browse(1)
        if st.real_time:
            timenow=time.strftime("%H:%M:%S")
            date=obj.date
            vals['time_stop']='%s %s'%(date,timenow)
        obj.write(vals)
        if context.get("called"):
            return obj.id
        return {
            'next': {
                'name': 'clinic_hd_case',
                'mode': 'form',
                'active_id': obj.id,
            },
            'flash': '%s is completed'%obj.number,
        }

    def delete(self,ids,context={}):
        for obj in self.browse(ids):
            if obj.state != 'draft':
                raise Exception("Can not delete HD Case %s because state is not draft"%obj.number)
        super().delete(ids)

    def onchange_hct(self,context={}):
        data=context['data']
        if not data.get("hct"):
            data['hct']=0
        hct=data["hct"]
        msg=""
        # XXX do not hard code
        if(hct<36):
            msg="สามารถเบิกค่ายาสูงสุดไม่เกิน 1,125บาท ต่อ สัปดาห์"
        elif(hct>=36 and hct<=39):
            msg="สามารถเบิกค่ายาสูงสุดไม่เกิน 750บาท ต่อ สัปดาห์"
        elif(hct> 39):
            msg="ไม่สามารถเบิกค่ายาฉีดได้ทุกตัว"
        data['hct_msg']=msg
        return data

    def undo(self,ids,context={}):
        obj=self.browse(ids)[0]
        for inv in obj.invoices:
            inv.write({
                'state': 'draft',
            })
            inv.delete()
        for pick in obj.pickings:
            pick.write({
                'state': 'draft',
            })
            pick.delete()
        for payment in obj.payments:
            payment.to_draft()
            payment.delete()
        for pm_line in obj.payment_lines:
            pm_line.delete()

        for exp in obj.expenes:
            exp.delete()

        state=context.get("state","in_progress") #force state
        obj.write({
            'state': state,
            'fee_paid': False,
        })

        return {
            'next': {
                'name': 'clinic_hd_case',
                'mode': 'form',
               'active_id': obj.id, 
            },
            'flash': '%s has been undo'%obj.number,
        }
    
    def view_payment(self,ids,context={}):
        print("clinic_view_payment")
        return {
            'next': {
                'name': 'payment',
                'mode': 'form',
                'active_id': ids[0],
            },
        }
    
    def request_fee(self,ids,context={}):
        obj=self.browse(ids)[0]
        obj.update_usetime()
        #if not obj.amount:
        obj.complete()
        # send some message to anyboby: patient
        return {
            'next': {
                'name': 'clinic_hd_case',
                'mode': 'form',
                'active_id': obj.id,
            }
        }

    def pay(self,ids,context={}):
        return {
            'next': {
                'name': 'clinic_payment',
                'refer_id': ids[0], #XXX
            }
        }

    def done(self,ids,context={}):
        obj=self.browse(ids)[0]
        obj.write({
            'state': 'completed',
        })
        return {
        'next': {
                'name': 'clinic_hd_case',
                'mode': 'form',
                'active_id': obj.id,
            },
            'flash': 'Finish treatment!',
        }
    
    def get_report_payment_data(self,context={}):
        settings=get_model("settings").browse(1)
        refer_id=context.get("refer_id")
        payment_id=context.get("payment_id")
        data={
            'settings_address_text': settings.default_address_id and settings.default_address_id.get_address_text()[settings.default_address_id.id] or "",
            'logo': get_file_path(settings.logo) or "",
        }
        if refer_id:
            pass
        if payment_id:
            #context['refer_id']=payment_id
            #data=get_model("account.payment").get_report_data(context=context)
            payment=get_model("account.payment").browse(int(payment_id)) 
            partner_address_id=payment.partner_id.default_address_id
            data['number']=payment.number
            data['ref']=payment.related_id.number
            data['date']=payment.date
            data['partner_name']=payment.partner_id.name or 0
            data['partner_address_text']=partner_address_id and partner_address_id.get_address_text()[partner_address_id.id] or "",
            lines=[]
            for line in payment.direct_lines:
                lines.append({
                    'description': line.description or '',
                    'qty': line.qty,
                    'unit_price': line.unit_price or 0.0,
                    'amount': line.amount or 0.0,
                })
            data['lines']=lines
            data['amount_subtotal']=payment.amount_subtotal or 0.0
            data['amount_tax']=payment.amount_tax or 0.0
            data['amount_total']=payment.amount_total or 0.0

        return data
    
    def new_dialyzer(self,ids,context={}):
        obj=self.browse(ids)[0]
        dlz_vals=get_model("clinic.dialyzer").default_get()
        dlz_vals['patient_id']=obj.patient_id.id
        dlz_vals['company_id']=dlz_vals['company_id'][0]
        product_name=dlz_vals['product_id'][1]
        dlz_vals['product_id']=dlz_vals['product_id'][0]
        dlz_id=get_model('clinic.dialyzer').create(dlz_vals)
        dialyzer=get_model("clinic.dialyzer").browse(dlz_id)
        dialyzer.confirm()
        vals={
            'dlz_id': dlz_id,
            'dialyzers': [],
        }
        vals['dialyzers'].append(('create',{
            'dialyzer_id': dlz_id,
            'description': dialyzer.description or product_name,
            'use_time': 1,
            'max_use_time': dialyzer.max_use_time,
            'member_type': dialyzer.member_type,
            'dialyzer_type': dialyzer.dialyzer_type,
            'bid_flow_rate': dialyzer.bid_flow_rate,
            'ultrafittration': dialyzer.ultrafittration,
            'state': dialyzer.state,
        }))
        obj.write(vals)
        if context.get('called'):
            return obj.id
        return {
            'next': {
                'name': 'clinic_hd_case',
                'mode': 'form',
                'active_id': obj.id,
            },
            'flash': 'Create new dialyzer successfully',
        }
    
    def to_draft(self,ids,context={}):
        obj=self.browse(ids)[0]
        context['state']='draft'
        obj.undo(context=context)
    
    def get_staff(self,ids,context={}):
        res={}
        for obj in self.browse(ids):
            doctor=0
            nurse=0
            doctor_id=None
            for ps in obj.staffs:
                if ps.type=="doctor":
                    if ps.priop=='owner':
                        doctor_id=ps.staff_id.id
                    doctor+= 1
                else:
                    nurse+=1
            res[obj.id]={
                'total_doctor': doctor,
                'total_nurse': nurse,
                'doctor_id': doctor_id,
            }
        return res
            

    def get_staff_fee(self,vals,patient_id=None):
        if not patient_id:
            return vals
        # staff
        patient=get_model("clinic.patient").browse(patient_id)
        doctor=patient.doctor_id
        if doctor:
            vals['staffs']=[]
            if not vals.get('staffs'):
                vals['staffs'].append(('create',{
                    'staff_id': doctor.id,
                    'type': 'doctor',
                    'priop': 'owner',
                }))
        # fee
        st=get_model("clinic.setting").browse(1)
        if not vals.get('lines'):
            vals['lines']=[]
            for st_prod in st.products:
                if patient.type_id.id==st_prod.patient_type_id.id:
                    prod=st_prod.product_id
                    price=st_prod.price
                    qty=st_prod.qty
                    amt=st_prod.amount
                    if not amt:
                        amt=qty*price
                    vals['lines'].append(('create',{
                        'product_id': prod.id,
                        'uom_id': st_prod.uom_id.id,
                        'type': st_prod.type,
                        'description': st_prod.description,
                        'price': price,
                        'qty': qty,
                        'amount': amt,
                    }))
        # XXX need to get default
        partner=patient.type_id.contact_id
        if partner:
            vals['fee_partner_id']=partner.id
        if not partner:
            raise Exception("Not found contact %s at menu: Patiens -> Type"%patient.type_id.name)
        return vals
    
    def get_invoice_policy(self,vals,patient_id=None):
        if patient_id:
            patient=get_model("clinic.patient").browse(patient_id)
            st=get_model("clinic.setting").browse(1)
            for pl in st.invoice_policies:
                policy=pl.invoice_policy
                patient_type_id=pl.patient_type_id.id
                opt=pl.invoice_option
                if patient.type_id.id==patient_type_id:
                    vals['invoice_policy']=policy
                    vals['invoice_option']=opt
                    break
        return vals

    def create(self,vals,**kw):
        patient_id=vals['patient_id']
        vals=self.get_staff_fee(vals,patient_id)
        vals=self.get_invoice_policy(vals,patient_id)
        new_id=super().create(vals,**kw)
        return new_id
    
    def write(self,ids,vals,**kw):
        patient_id=vals.get('patient_id')
        vals=self.get_invoice_policy(vals,patient_id)
        obj=self.browse(ids)[0]
        if not obj.amount:
            vals['req_fee']=1 # to show button pay
        super().write(ids,vals,**kw)

    def approve(self,ids,context={}):
        obj=self.browse(ids)[0]
        obj.write({
            'state': 'completed',
        })
        return {
            'next': {
                'name': 'clinic_hd_case',
                'mode': 'form',
                'active_id': obj.id,
            },
            'flash': '%s has been approval'%obj.number,
        }


HDCase.register()
